from django.apps import AppConfig


class DjangoVectorstoreIndexModelConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "llmhelper.extra.django_vectorstore_index_model"
